import abc
from typing import Iterable

from .hashpoint import *
from .mentionable import *

__all__ = ('RecursiveMentionable',)


class RecursiveMentionable(Mentionable, abc.ABC):
    def points(self) -> Iterable[HashPoint]:
        raise NotImplementedError

    @classmethod
    def points_of(cls, mentionable: Mentionable) -> Iterable[HashPoint]:
        if isinstance(mentionable, RecursiveMentionable):
            return mentionable.points()
        else:
            return []

    async def str(self, tab: int) -> str:
        assert isinstance(tab, int)
        return f'(recursive {self.__class__.__name__})'

    def __topology_hash__(self) -> bytes:
        return HashPoint.hash(b''.join(hash_point.point for hash_point in self.points()))
