from typing import Any, Callable, Coroutine, Generic, TypeVar

__all__ = ('Mapper',)

Element = TypeVar('Element')
Mapped = TypeVar('Mapped')


class Mapper(Generic[Element, Mapped]):
    async def map(self, element: Element) -> Mapped:
        raise NotImplementedError

    def bind(self, target: Callable[[], Coroutine[Any, Any, Element]]) -> Callable[[], Coroutine[Any, Any, Mapped]]:
        async def bound() -> Mapped:
            return await self.map(await target())

        return bound
