from typing import Generic, Iterable, Optional, TypeVar

from rainbowadn.core import *
from .inlining import *

__all__ = ('IRef',)

TRef = TypeVar('TRef')


class IRef(RecursiveMentionable, Generic[TRef]):
    def points(self) -> Iterable[HashPoint]:
        return [self.hashpoint]

    def __bytes__(self):
        return bytes(self.hashpoint)

    def __factory__(self) -> RainbowFactory['IRef[TRef]']:
        return IReff(self.hashpoint.factory)

    def __init__(self, hashpoint: HashPoint[TRef]):
        assert isinstance(hashpoint, HashPoint)
        self.hashpoint = hashpoint


class IReff(Inlining[IRef[TRef]], Generic[TRef]):
    def __init__(self, factory: RainbowFactory[TRef]):
        assert isinstance(factory, RainbowFactory)
        self.factory = factory

    def size(self) -> Optional[int]:
        return HashPoint.HASH_LENGTH

    def from_bytes(self, source: bytes, resolver: HashResolver) -> IRef[TRef]:
        assert_eq(len(source), HashPoint.HASH_LENGTH)
        return IRef(
            ResolverMetaOrigin(resolver).hash_point(self.factory, source)
        )
