import unittest

from rainbowadn.atomic import *
from rainbowadn.core import *
from rainbowadn.inlining import *
from .resolvers import *


class TestInlining(unittest.IsolatedAsyncioTestCase):
    @classmethod
    async def _format(cls, *hps: HashPoint):
        result = []
        for hp in hps:
            result.append(await hash_point_format(hp, 0))
        return result

    async def test_unit(self):
        print(await (IAuto.auto_f(IUnit.factory(), IUnit.factory()).from_bytes(b'', FailResolver())).str(0))
        print(bytes(IAuto.auto_of(IUnit(), IUnit())))

    async def test_byte(self):
        # iB short for IByte
        print(await (IAuto.auto_f(IByte.factory(), IByte.factory()).from_bytes(b'iB', FailResolver())).str(0))
        print(bytes(IAuto.auto_of(IByte(0x69), IByte(0x42))))

    async def test_mixed(self):
        print(await (IAuto.auto_f(IByte.factory(), IUnit.factory()).from_bytes(b'!', FailResolver())).str(0))
        print(bytes(IAuto.auto_of(IByte(0x21), IUnit())))

    async def test_complex_infix(self):
        print(b := bytes(IAuto.auto_of(IByte(0x21), Integer(0x21), IByte(0x21))))
        print(
            await IAuto.auto_f(
                IByte.factory(), Integer.factory(), IByte.factory()
            ).from_bytes(b, FailResolver()).str(0)
        )

    async def test_complex_reference(self):
        dr = default_resolver()
        print(
            (
                b := bytes(
                    await dr.migrate_resolved(
                        IAuto.auto_of(IByte(0x21), Integer(0x21), Integer(0x21), IByte(0x21))
                    )
                )
            ).hex()
        )
        print(
            await IAuto.auto_f(
                IByte.factory(), Integer.factory(), Integer.factory(), IByte.factory()
            ).from_bytes(b, dr).str(0)
        )

    async def test_pair(self):
        with self.subTest('of'):
            pair = IPair.of(IUnit(), IUnit())
            print(await pair.str(0))
            print(bytes(pair).hex())
        with self.subTest('off'):
            pair = await IPair.off(HashPoint.of(IUnit()), HashPoint.of(IUnit()))
            print(await pair.str(0))
            print(bytes(pair).hex())
        with self.subTest('integer'):
            pair = IPair.of(Integer(0x21), IUnit())
            print(await pair.str(0))
            print(bytes(pair).hex())
        with self.subTest('integer 2'):
            pair = IPair.of(Integer(0x21), Integer(0x21))
            print(await pair.str(0))
            print(bytes(pair).hex())
